mod inbox;
mod users;
mod webfinger;

use axum::{
    Router,
    http::{StatusCode, Uri},
    routing::{get, post},
};
use inbox::inbox_post;
use users::users_get;
use webfinger::webfinger_get;

const HOST: &'static str = "testing4host.com";
const USER: &'static str = "test71";

async fn fallback(uri: Uri) -> (StatusCode, String) {
    println!("Unknown url! {}", uri);
    (StatusCode::NOT_FOUND, format!("No route for {uri}"))
}

#[tokio::main]
async fn main() {
    env_logger::init();

    let app = Router::new()
        .route("/", get(root))
        .route("/.well-known/webfinger", get(webfinger_get))
        .route("/users/{users}/", get(users_get))
        .route("/users/{users}", get(users_get))
        .route("/inbox/{users}/", post(inbox_post))
        .fallback(fallback);

    let listener = tokio::net::TcpListener::bind("0.0.0.0:8000").await.unwrap();
    axum::serve(listener, app).await.unwrap();
}

async fn root() -> &'static str {
    "Hello World!"
}
